/*
 * Copyright (c) 2008 Pavel Rimsky
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup sparc64
 * @{
 */
/**
 * @file
 * @brief	Niagara input/output driver based on hypervisor calls.
 */

#include <arch/drivers/niagara.h>
#include <console/chardev.h>
#include <console/console.h>
#include <arch/sun4v/hypercall.h>

/* functions referenced from definitions of I/O operations structures */
static void niagara_putchar(chardev_t *, const char);

/** character device operations */
static chardev_operations_t niagara_ops = {
	.write = niagara_putchar
};

/** Niagara character device */
chardev_t niagara_io;

/** Writes a single character to the standard output. */
static void niagara_putchar(struct chardev * cd, const char c)
{
	__hypercall_fast1(CONS_PUTCHAR, c);
	if (c == '\n')
		__hypercall_fast1(CONS_PUTCHAR, '\r');
}


/**
 * Grabs the input for kernel.
 */
void niagara_grab(void)
{
}

/**
 * Releases the input so that userspace can use it.
 */
void niagara_release(void)
{
}

/**
 * Initializes the input/output subsystem so that the Niagara standard
 * input/output is used.
 */
void niagara_init(void)
{
	chardev_initialize("niagara_io", &niagara_io, &niagara_ops);
	stdin = &niagara_io;
	stdout = &niagara_io;
}

/** @}
 */
